"""
logtracker.py
by Jason Conti
February 15, 2010

Tracks a log file for changes.
"""

import os
import os.path
import pyinotify
import time

class LogTracker(pyinotify.ProcessEvent):
  """Tracks a log file for changes."""
  def __init__(self, path):
    pyinotify.ProcessEvent.__init__(self)

    self._path = os.path.abspath(path)
    self._offset = self._get_file_length()

    self._listeners = set()

  def add_listener(self, f):
    """A listener is a function of a single argument, the text appended to
    the log when it changed."""
    self._listeners.add(f)

  def remove_listener(self, f):
    """Removes f from the set of listeners."""
    self._listeners.remove(f)

  def start(self):
    """Starts watching the file."""
    self._wm = pyinotify.WatchManager()
    self._wdd = self._wm.add_watch(self._path, pyinotify.IN_MODIFY)
    self._notifier = pyinotify.ThreadedNotifier(self._wm, self)
    self._notifier.start()

  def stop(self):
    """Removes the watch from the tracked file and stops the notifier."""
    self._wm.rm_watch(self._wdd.values())
    self._notifier.stop()

  def _get_file_length(self):
    """Returns the length of the tracked file, returns 0 if there is an error,
    assuming that the file may not have been created yet."""
    try:
      f = open(self._path, "r")
      f.seek(0, os.SEEK_END)
      size = f.tell()
      f.close()
      return size
    except:
      return 0

  def _read_appended_text(self):
    """Opens the tracked file, seeks to the last read offset, reads to EOF
    and updates the last read offset."""
    f = open(self._path, "r")
    f.seek(self._offset, os.SEEK_SET)
    text = f.read()
    self._offset = f.tell()
    f.close()
    return text

  def process_IN_MODIFY(self, event):
    """Reads the appended text in the tracked file and sends it to the listeners."""
    text = self._read_appended_text()
    for f in self._listeners:
      f(text)

def test1(text):
  print "test1:"
  print text

def test2(text):
  print "test2:"
  print text

def main():
  track = LogTracker("test.log")
  track.add_listener(test1)
  track.add_listener(test2)

  while True:
    try:
      time.sleep(10000)
    except:
      track.stop()
      break

if __name__ == '__main__':
  main()

